@@ -1,21 +1,21 @@
 #include <RcppArmadillo.h>
 // [[Rcpp::depends(RcppArmadillo)]]
 
-#ifdef _OPENMP
-#include <omp.h>
-#endif
-
-
-#include "thread_num.h"
+// #ifdef _OPENMP
+// #include <omp.h>
+// #endif
+//
+//
+// #include "thread_num.h"
 
 using namespace Rcpp;
 
-typedef std::vector<arma::mat> stdvec_mat;
+// typedef std::vector<arma::mat> stdvec_mat;
 
 
 
 // [[Rcpp::export]]
-long double ldwnorm2_num( arma::vec x, arma::vec par,  arma::mat omega_2pi) {
+double ldwnorm2_num( arma::vec x, arma::vec par,  arma::mat omega_2pi) {
   int M = omega_2pi.n_rows;
   arma::vec allexpons(M);
   double expon_sum = 0.0, term1, term2;
@@ -30,6 +30,8 @@ long double ldwnorm2_num( arma::vec x, arma::vec par,  arma::mat omega_2pi) {
 }
 
 
+
+
 // [[Rcpp::export]]
 double l_const_wnorm2(arma::vec par) {
   double det_sig_inv = par[0] * par[1] - par[2]*par[2];
@@ -61,7 +63,6 @@ arma::mat mem_p_wnorm2(arma::mat data, arma::mat par_mat, arma::vec pi,
   double row_total;
   arma::mat den(n, K);
   arma::vec log_pi = log(pi);
-#pragma omp parallel for private(j, row_total) num_threads(ncores)
   for(int i = 0; i < n; i++) {
     row_total = 0;
     for(j = 0; j < K; j++){
@@ -76,34 +77,148 @@ arma::mat mem_p_wnorm2(arma::mat data, arma::mat par_mat, arma::vec pi,
 }
 
 
+// // not used
+// // [[Rcpp::export]]
+// double llik_wnorm2_full(arma::mat data, arma::mat par, arma::vec pi,
+//                              arma::vec log_c, arma::mat omega_2pi, int ncores = 1)
+// {
+//   int n = data.n_rows, K = pi.size(), j;
+//   double temp, log_sum = 0.0;
+//   arma::vec log_pi = log(pi);
+//
+//   if(K > 1) {
+//     for(int i = 0; i < n; i++) {
+//       temp = 0;
+//       for(j = 0; j < K; j++)
+//         temp += exp(ldwnorm2_num(arma::trans(data.row(i)), par.col(j), omega_2pi) - log_c[j] + log_pi[j] );
+//       log_sum += log(temp);
+//     }
+//   } else {
+//     for(int i = 0; i < n; i++) {
+//       log_sum += ldwnorm2_num(arma::trans(data.row(i)), par, omega_2pi);
+//     }
+//     log_sum -= n*log_c[0];
+//   }
+//   return(log_sum);
+// }
+//
+
 
 // [[Rcpp::export]]
-long double llik_wnorm2_full(arma::mat data, arma::mat par, arma::vec pi,
-                             arma::vec log_c, arma::mat omega_2pi, int ncores = 1)
+arma::vec llik_wnorm2_contri_C(arma::mat data, arma::mat par, arma::vec pi,
+                               arma::vec log_c, arma::mat omega_2pi)
 {
   int n = data.n_rows, K = pi.size(), j;
-  long double temp, log_sum = 0.0;
+  double temp;
   arma::vec log_pi = log(pi);
+  arma::vec llik_contri = arma::vec(n);
+
 
   if(K > 1) {
-#pragma omp parallel for reduction(+:log_sum)  private(j, temp) num_threads(ncores)
     for(int i = 0; i < n; i++) {
       temp = 0;
       for(j = 0; j < K; j++)
         temp += exp(ldwnorm2_num(arma::trans(data.row(i)), par.col(j), omega_2pi) - log_c[j] + log_pi[j] );
-      log_sum += log(temp);
+      llik_contri[i] = log(temp);
     }
   } else {
-#pragma omp parallel for reduction(+:log_sum)  num_threads(ncores)
     for(int i = 0; i < n; i++) {
-      log_sum += ldwnorm2_num(arma::trans(data.row(i)), par, omega_2pi);
+      llik_contri[i] = ldwnorm2_num(arma::trans(data.row(i)),
+                                    par, omega_2pi) - log_c[0];
     }
-    log_sum -= n*log_c[0];
   }
-  return(log_sum);
+
+  return(llik_contri);
 }
 
 
+
+
+// [[Rcpp::export]]
+double llik_wnorm2_one_comp(arma::mat data, arma::vec par_vec,
+                            double log_c, arma::mat omega_2pi)
+{
+  int n = data.n_rows;
+  double log_sum = 0.0;
+
+  for(int i = 0; i < n; i++) {
+    log_sum += ldwnorm2_num(arma::trans(data.row(i)), par_vec, omega_2pi);
+  }
+  log_sum -= n*log_c;
+
+  return log_sum;
+}
+
+
+
+// // [[Rcpp::export]]
+arma::vec grad_log_den_wnorm2_1_comp_1_point(double x, double y,
+                                             double s11, double s22, double s12,
+                                             double mu1, double mu2,
+                                             double det_prec,
+                                             arma::mat omega_2pi)
+{
+  int M = omega_2pi.n_rows;
+  double expon_j, term1, term2;
+  arma::vec sum_entries = arma::zeros(6);
+  for(int j = 0; j < M; j++) {
+    term1 = (omega_2pi(j,0) - x + mu1);
+    term2 = (omega_2pi(j,1) - y + mu2);
+    expon_j = exp(-0.5 *( s11 * term1 * term1 + s22 * term2 * term2 +  2 * s12 * term1 * term2 ));
+
+    sum_entries[0] += expon_j * (s22 - det_prec * term1 * term1 ) * 0.5 / det_prec;
+    sum_entries[1] += expon_j * (s11 - det_prec * term2 * term2) * 0.5 / det_prec;
+    sum_entries[2] += expon_j * (-s12 - det_prec * term1 * term2 ) / det_prec;
+    sum_entries[3] += expon_j * (- s11 * term1 - s12 * term2 );
+    sum_entries[4] += expon_j * (- s12 * term1 - s22 * term2 );
+    sum_entries[5] += expon_j ;
+  }
+
+  // det_prec = det(precision matrix) = (k1k2 -k3^2)
+
+  // Now divide all the 1st 5 terms by the last sum to get grad(log density)
+  for(int j = 0; j < 5; j++) {
+    sum_entries[j] /= sum_entries[5];
+  }
+
+  // finally return log of the last term (log density numerator)
+  sum_entries[5] = log(sum_entries[5]);
+
+  return sum_entries;
+}
+
+
+
+
+
+
+// [[Rcpp::export]]
+arma::vec grad_llik_wnorm2_C(arma::mat data, arma::vec par, arma::mat omega_2pi) {
+  int n = data.n_rows;
+  arma::vec grad_llik_wo_const = arma::zeros(6);
+  // first 5 entries = grad, last entry = llik
+  double  s11 = par[0], s22 = par[1], s12 = par[2], mu1 = par[3], mu2 = par[4];
+
+  double det_prec = s11*s22 - s12*s12;
+
+
+  for(int i = 0; i < n; i++) {
+    grad_llik_wo_const +=
+      grad_log_den_wnorm2_1_comp_1_point(data(i, 0), data(i, 1),
+                                         s11, s22, s12, mu1, mu2,
+                                         det_prec,
+                                         omega_2pi);
+  }
+
+  // Now adjust the constant in llik
+  grad_llik_wo_const[5] -= n*(log(2*M_PI) - 0.5*log(det_prec));
+
+  return grad_llik_wo_const;
+}
+
+
+
+
 // [[Rcpp::export]]
 arma::vec grad_den_wnorm2_one_comp_i_unadj(double x, double y, arma::vec par, double det_sig_inv,
                                            double det_sig_inv_sqrt, arma::mat omega_2pi)
@@ -133,79 +248,79 @@ arma::vec grad_den_wnorm2_one_comp_i_unadj(double x, double y, arma::vec par, do
 }
 
 
-// [[Rcpp::export]]
-arma::mat grad_wnorm2_all_comp(arma::mat data, arma::mat par_mat, arma::vec pi,
-                               arma::mat omega_2pi, int ncores = 1)
-{
-  int n = data.n_rows, K = pi.size(), j;
-  double denom;
-  arma::mat grad_sum = arma::zeros(5, K);
-
-  stdvec_mat grad_sum_part(ncores);
-  for(int g = 0; g < ncores; g++)
-    grad_sum_part[g] = grad_sum;
-
-  arma::vec det_sig_inv(K);
-  for(j = 0; j < K; j++)
-    det_sig_inv[j] = par_mat(0, j) * par_mat(1, j) - par_mat(2, j) * par_mat(2, j);
-
-  arma::vec det_sig_inv_sqrt = arma::sqrt(det_sig_inv);
-
-#pragma omp parallel for   private(j, denom) num_threads(ncores)
-  for(int i = 0; i < n; i++) {
-    arma::mat grad_den_temp(6, K);
-    int g = get_thread_num_final();
-    // int g = 0;
-    denom = 0;
-    for(j = 0; j < K; j++){
-      // denom += exp(ldwnorm2_num(trans(data.row(i)), par_mat.col(j), omega_2pi) - log_wnorm2_const[j] + log_pi[j] );
-      grad_den_temp.col(j) = grad_den_wnorm2_one_comp_i_unadj(data(i, 0), data(i, 1), par_mat.col(j),
-                        det_sig_inv[j], det_sig_inv_sqrt[j], omega_2pi);
-      denom += grad_den_temp(5, j) * det_sig_inv_sqrt[j] * pi[j];
-    }
-    grad_sum_part[g] += grad_den_temp.head_rows(5) / denom;
-  }  // the pi_h term in numerator hasn't been adjusted yet!
-
-  for(int g = 0; g < ncores; g++)
-    grad_sum += grad_sum_part[g];
-
-  arma::mat adj_factor(5, K);
-  for(j = 0; j < K; j++){
-    adj_factor(0,j) = pi[j] / (2 * det_sig_inv_sqrt[j]);
-    adj_factor(1,j) = pi[j] / (2 * det_sig_inv_sqrt[j]);
-    adj_factor(2,j) = pi[j] / det_sig_inv_sqrt[j];
-    adj_factor(3,j) = pi[j] * det_sig_inv_sqrt[j];
-    adj_factor(4,j) = pi[j] * det_sig_inv_sqrt[j];
-  }
-
-  return (grad_sum % adj_factor);
-}
+// // not used
+// // [[Rcpp::export]]
+// arma::mat grad_wnorm2_all_comp(arma::mat data, arma::mat par_mat, arma::vec pi,
+//                                arma::mat omega_2pi, int ncores = 1)
+// {
+//   int n = data.n_rows, K = pi.size(), j;
+//   double denom;
+//   arma::mat grad_sum = arma::zeros(5, K);
+//
+//   stdvec_mat grad_sum_part(ncores);
+//   for(int g = 0; g < ncores; g++)
+//     grad_sum_part[g] = grad_sum;
+//
+//   arma::vec det_sig_inv(K);
+//   for(j = 0; j < K; j++)
+//     det_sig_inv[j] = par_mat(0, j) * par_mat(1, j) - par_mat(2, j) * par_mat(2, j);
+//
+//   arma::vec det_sig_inv_sqrt = arma::sqrt(det_sig_inv);
+//
+//   for(int i = 0; i < n; i++) {
+//     arma::mat grad_den_temp(6, K);
+//     int g = get_thread_num_final();
+//     // int g = 0;
+//     denom = 0;
+//     for(j = 0; j < K; j++){
+//       // denom += exp(ldwnorm2_num(trans(data.row(i)), par_mat.col(j), omega_2pi) - log_wnorm2_const[j] + log_pi[j] );
+//       grad_den_temp.col(j) = grad_den_wnorm2_one_comp_i_unadj(data(i, 0), data(i, 1), par_mat.col(j),
+//                         det_sig_inv[j], det_sig_inv_sqrt[j], omega_2pi);
+//       denom += grad_den_temp(5, j) * det_sig_inv_sqrt[j] * pi[j];
+//     }
+//     grad_sum_part[g] += grad_den_temp.head_rows(5) / denom;
+//   }  // the pi_h term in numerator hasn't been adjusted yet!
+//
+//   for(int g = 0; g < ncores; g++)
+//     grad_sum += grad_sum_part[g];
+//
+//   arma::mat adj_factor(5, K);
+//   for(j = 0; j < K; j++){
+//     adj_factor(0,j) = pi[j] / (2 * det_sig_inv_sqrt[j]);
+//     adj_factor(1,j) = pi[j] / (2 * det_sig_inv_sqrt[j]);
+//     adj_factor(2,j) = pi[j] / det_sig_inv_sqrt[j];
+//     adj_factor(3,j) = pi[j] * det_sig_inv_sqrt[j];
+//     adj_factor(4,j) = pi[j] * det_sig_inv_sqrt[j];
+//   }
+//
+//   return (grad_sum % adj_factor);
+// }
 
 
 
-
-// [[Rcpp::export]]
-long double wnorm2mix(arma::vec x, arma::mat par, arma::vec pi,
-                      arma::vec log_wnorm_const, arma::mat omega_2pi) {
-  long double res = 0;
-  int K = pi.size();
-
-  for(int j = 0; j < K; j++)
-    res += exp(ldwnorm2_num(x, par.col(j), omega_2pi) - log_wnorm_const[j]) * pi[j];
-  return(res);
-
-}
-
-// [[Rcpp::export]]
-arma::vec wnorm2mix_manyx(arma::mat x, arma::mat par, arma::vec pi,
-                          arma::vec log_c_von, arma::mat omega_2pi)
-{
-  int n = x.n_rows;
-  arma::vec result(n);
-  for(int i = 0; i < n; i++)
-    result[i] = wnorm2mix(arma::trans(x.row(i)), par, pi, log_c_von, omega_2pi);
-  return(result);
-}
+// // not used
+// // [[Rcpp::export]]
+// double wnorm2mix(arma::vec x, arma::mat par, arma::vec pi,
+//                       arma::vec log_wnorm_const, arma::mat omega_2pi) {
+//   double res = 0;
+//   int K = pi.size();
+//
+//   for(int j = 0; j < K; j++)
+//     res += exp(ldwnorm2_num(x, par.col(j), omega_2pi) - log_wnorm_const[j]) * pi[j];
+//   return(res);
+//
+// }
+//
+// // [[Rcpp::export]]
+// arma::vec wnorm2mix_manyx(arma::mat x, arma::mat par, arma::vec pi,
+//                           arma::vec log_c_von, arma::mat omega_2pi)
+// {
+//   int n = x.n_rows;
+//   arma::vec result(n);
+//   for(int i = 0; i < n; i++)
+//     result[i] = wnorm2mix(arma::trans(x.row(i)), par, pi, log_c_von, omega_2pi);
+//   return(result);
+// }
 
 
 // [[Rcpp::export]]
